@@ -31,6 +31,7 @@ import java.lang.reflect.Constructor;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -106,6 +107,7 @@ static
 
 
 	dispatchMacros['^'] = new MetaReader();
+	dispatchMacros['#'] = new SymbolicValueReader();
 	dispatchMacros['\''] = new VarReader();
 	dispatchMacros['"'] = new RegexReader();
 	dispatchMacros['('] = new FnReader();
@@ -115,8 +117,16 @@ static
 	dispatchMacros['<'] = new UnreadableReader();
 	dispatchMacros['_'] = new DiscardReader();
 	dispatchMacros['?'] = new ConditionalReader();
+	dispatchMacros[':'] = new NamespaceMapReader();
 	}
 
+public static interface Resolver{
+    Symbol currentNS();
+    Symbol resolveClass(Symbol sym);
+    Symbol resolveAlias(Symbol sym);
+    Symbol resolveVar(Symbol sym);
+}
+
 static boolean isWhitespace(int ch){
 	return Character.isWhitespace(ch) || ch == ',';
 }
@@ -193,11 +203,11 @@ static public Object read(PushbackReader r, boolean eofIsError, Object eofValue,
 static public Object read(PushbackReader r, boolean eofIsError, Object eofValue, boolean isRecursive, Object opts)
 {
 	// start with pendingForms null as reader conditional splicing is not allowed at top level
-	return read(r, eofIsError, eofValue, null, null, isRecursive, opts, null);
+	return read(r, eofIsError, eofValue, null, null, isRecursive, opts, null, (Resolver) RT.READER_RESOLVER.deref());
 }
 
 static private Object read(PushbackReader r, boolean eofIsError, Object eofValue, boolean isRecursive, Object opts, Object pendingForms) {
-	return read(r, eofIsError, eofValue, null, null, isRecursive, opts, ensurePending(pendingForms));
+	return read(r, eofIsError, eofValue, null, null, isRecursive, opts, ensurePending(pendingForms), (Resolver) RT.READER_RESOLVER.deref());
 }
 
 static private Object ensurePending(Object pendingForms) {
@@ -220,7 +230,9 @@ static private Object installPlatformFeature(Object opts) {
     }
 }
 
-static private Object read(PushbackReader r, boolean eofIsError, Object eofValue, Character returnOn, Object returnOnValue, boolean isRecursive, Object opts, Object pendingForms)
+static private Object read(PushbackReader r, boolean eofIsError, Object eofValue, Character returnOn,
+                           Object returnOnValue, boolean isRecursive, Object opts, Object pendingForms,
+                           Resolver resolver)
 {
     if(RT.READEVAL.deref() == UNKNOWN)
         throw Util.runtimeException("Reading disallowed - *read-eval* bound to :unknown");
@@ -280,7 +292,7 @@ static private Object read(PushbackReader r, boolean eofIsError, Object eofValue
 				}
 
 			String token = readToken(r, (char) ch);
-			return interpretToken(token);
+			return interpretToken(token, resolver);
 			}
 		}
 	catch(Exception e)
@@ -368,7 +380,7 @@ static private int readUnicodeChar(PushbackReader r, int initch, int base, int l
 	return uc;
 }
 
-static private Object interpretToken(String s) {
+static private Object interpretToken(String s, Resolver resolver) {
 	if(s.equals("nil"))
 		{
 		return null;
@@ -383,7 +395,7 @@ static private Object interpretToken(String s) {
 		}
 	Object ret = null;
 
-	ret = matchSymbol(s);
+	ret = matchSymbol(s, resolver);
 	if(ret != null)
 		return ret;
 
@@ -391,7 +403,7 @@ static private Object interpretToken(String s) {
 }
 
 
-private static Object matchSymbol(String s){
+private static Object matchSymbol(String s, Resolver resolver){
 	Matcher m = symbolPat.matcher(s);
 	if(m.matches())
 		{
@@ -405,17 +417,33 @@ private static Object matchSymbol(String s){
 		if(s.startsWith("::"))
 			{
 			Symbol ks = Symbol.intern(s.substring(2));
-			Namespace kns;
-			if(ks.ns != null)
-				kns = Compiler.namespaceFor(ks);
-			else
-				kns = Compiler.currentNS();
-			//auto-resolving keyword
-			if (kns != null)
-				return Keyword.intern(kns.name.name,ks.name);
-			else
-				return null;
-			}
+            if(resolver != null)
+                {
+                Symbol nsym;
+                if(ks.ns != null)
+                    nsym = resolver.resolveAlias(Symbol.intern(ks.ns));
+                else
+                    nsym = resolver.currentNS();
+                //auto-resolving keyword
+                if(nsym != null)
+                    return Keyword.intern(nsym.name, ks.name);
+                else
+                    return null;
+                }
+            else
+                {
+                Namespace kns;
+                if(ks.ns != null)
+                    kns = Compiler.currentNS().lookupAlias(Symbol.intern(ks.ns));
+                else
+                    kns = Compiler.currentNS();
+                //auto-resolving keyword
+                if(kns != null)
+                    return Keyword.intern(kns.name.name, ks.name);
+                else
+                    return null;
+                }
+            }
 		boolean isKeyword = s.charAt(0) == ':';
 		Symbol sym = Symbol.intern(s.substring(isKeyword ? 1 : 0));
 		if(isKeyword)
@@ -597,6 +625,130 @@ public static class DiscardReader extends AFn{
 	}
 }
 
+// :a.b{:c 1} => {:a.b/c 1}
+// ::{:c 1}   => {:a.b/c 1}  (where *ns* = a.b)
+// ::a{:c 1}  => {:a.b/c 1}  (where a is aliased to a.b)
+public static class NamespaceMapReader extends AFn{
+	public Object invoke(Object reader, Object colon, Object opts, Object pendingForms) {
+		PushbackReader r = (PushbackReader) reader;
+
+		boolean auto = false;
+		int autoChar = read1(r);
+		if(autoChar == ':')
+			auto = true;
+		else
+		    unread(r, autoChar);
+
+		Object sym = null;
+		int nextChar = read1(r);
+		if(isWhitespace(nextChar)) {  // the #:: { } case or an error
+			if(auto) {
+				while (isWhitespace(nextChar))
+					nextChar = read1(r);
+				if(nextChar != '{') {
+					unread(r, nextChar);
+					throw Util.runtimeException("Namespaced map must specify a namespace");
+				}
+			} else {
+				unread(r, nextChar);
+				throw Util.runtimeException("Namespaced map must specify a namespace");
+			}
+		} else if(nextChar != '{') {  // #:foo { } or #::foo { }
+			unread(r, nextChar);
+			sym = read(r, true, null, false, opts, pendingForms);
+			nextChar = read1(r);
+			while(isWhitespace(nextChar))
+				nextChar = read1(r);
+		}
+		if(nextChar != '{')
+			throw Util.runtimeException("Namespaced map must specify a map");
+
+		// Resolve autoresolved ns
+		String ns;
+		if (auto) {
+            Resolver resolver = (Resolver) RT.READER_RESOLVER.deref();
+			if (sym == null) {
+                if(resolver != null)
+                    ns = resolver.currentNS().name;
+                else
+				    ns = Compiler.currentNS().getName().getName();
+			} else if (!(sym instanceof Symbol) || ((Symbol)sym).getNamespace() != null) {
+				throw Util.runtimeException("Namespaced map must specify a valid namespace: " + sym);
+			} else {
+				Symbol resolvedNS;
+				if (resolver != null)
+                    resolvedNS = resolver.resolveAlias((Symbol) sym);
+				else{
+                    Namespace rns = Compiler.currentNS().lookupAlias((Symbol)sym);
+                    resolvedNS = rns != null?rns.getName():null;
+                }
+
+				if(resolvedNS == null) {
+					throw Util.runtimeException("Unknown auto-resolved namespace alias: " + sym);
+				} else {
+					ns = resolvedNS.getName();
+				}
+			}
+		} else if (!(sym instanceof Symbol) || ((Symbol)sym).getNamespace() != null) {
+			throw Util.runtimeException("Namespaced map must specify a valid namespace: " + sym);
+		} else {
+			ns = ((Symbol)sym).getName();
+		}
+
+		// Read map
+		List kvs = readDelimitedList('}', r, true, opts, ensurePending(pendingForms));
+		if((kvs.size() & 1) == 1)
+			throw Util.runtimeException("Namespaced map literal must contain an even number of forms");
+
+		// Construct output map
+		Object[] a = new Object[kvs.size()];
+		Iterator iter = kvs.iterator();
+		for(int i = 0; iter.hasNext(); i += 2) {
+			Object key = iter.next();
+			Object val = iter.next();
+
+			if(key instanceof Keyword) {
+				Keyword kw = (Keyword) key;
+				if (kw.getNamespace() == null) {
+					key = Keyword.intern(ns, kw.getName());
+				} else if (kw.getNamespace().equals("_")) {
+					key = Keyword.intern(null, kw.getName());
+				}
+			} else if(key instanceof Symbol) {
+				Symbol s = (Symbol) key;
+				if (s.getNamespace() == null) {
+					key = Symbol.intern(ns, s.getName());
+				} else if (s.getNamespace().equals("_")) {
+					key = Symbol.intern(null, s.getName());
+				}
+			}
+			a[i] = key;
+			a[i+1] = val;
+		}
+		return RT.map(a);
+	}
+}
+
+
+public static class SymbolicValueReader extends AFn{
+
+    static IPersistentMap  specials = PersistentHashMap.create(Symbol.intern("Inf"), Double.POSITIVE_INFINITY,
+                                                               Symbol.intern("-Inf"), Double.NEGATIVE_INFINITY,
+                                                               Symbol.intern("NaN"), Double.NaN);
+
+	public Object invoke(Object reader, Object quote, Object opts, Object pendingForms) {
+		PushbackReader r = (PushbackReader) reader;
+		Object o = read(r, true, null, true, opts, ensurePending(pendingForms));
+
+		if (!(o instanceof Symbol))
+			throw Util.runtimeException("Invalid token: ##" + o);
+		if (!(specials.containsKey(o)))
+			throw Util.runtimeException("Unknown symbolic value: ##" + o);
+
+		return specials.valAt(o);
+	}
+}
+
 public static class WrappingReader extends AFn{
 	final Symbol sym;
 
@@ -760,7 +912,7 @@ static class ArgReader extends AFn{
 		PushbackReader r = (PushbackReader) reader;
 		if(ARG_ENV.deref() == null)
 			{
-			return interpretToken(readToken(r, '%'));
+			return interpretToken(readToken(r, '%'), null);
 			}
 		int ch = read1(r);
 		unread(r, ch);
@@ -845,6 +997,7 @@ public static class SyntaxQuoteReader extends AFn{
 			ret = RT.list(Compiler.QUOTE, form);
 		else if(form instanceof Symbol)
 			{
+            Resolver resolver = (Resolver) RT.READER_RESOLVER.deref();
 			Symbol sym = (Symbol) form;
 			if(sym.ns == null && sym.name.endsWith("#"))
 				{
@@ -861,13 +1014,43 @@ public static class SyntaxQuoteReader extends AFn{
 			else if(sym.ns == null && sym.name.endsWith("."))
 				{
 				Symbol csym = Symbol.intern(null, sym.name.substring(0, sym.name.length() - 1));
-				csym = Compiler.resolveSymbol(csym);
+				if(resolver != null){
+                    Symbol rc = resolver.resolveClass(csym);
+                    if(rc != null)
+                        csym = rc;
+                }
+				else
+				    csym = Compiler.resolveSymbol(csym);
 				sym = Symbol.intern(null, csym.name.concat("."));
 				}
 			else if(sym.ns == null && sym.name.startsWith("."))
 				{
 				// Simply quote method names.
 				}
+            else if(resolver != null)
+                {
+                Symbol nsym = null;
+                if(sym.ns != null){
+                    Symbol alias = Symbol.intern(null, sym.ns);
+                    nsym = resolver.resolveClass(alias);
+                    if(nsym == null)
+                        nsym = resolver.resolveAlias(alias);
+                    }
+                if(nsym != null){
+                    // Classname/foo -> package.qualified.Classname/foo
+                    sym = Symbol.intern(nsym.name, sym.name);
+                    }
+                else if(sym.ns == null){
+                    Symbol rsym = resolver.resolveClass(sym);
+                    if(rsym == null)
+                        rsym = resolver.resolveVar(sym);
+                    if(rsym != null)
+                        sym = rsym;
+                    else
+                        sym = Symbol.intern(resolver.currentNS().name,sym.name);
+                }
+                //leave alone if qualified
+                }
 			else
 				{
 				Object maybeClass = null;
@@ -1194,10 +1377,12 @@ public static List readDelimitedList(char delim, PushbackReader r, boolean isRec
 			((LineNumberingPushbackReader) r).getLineNumber() : -1;
 
 	ArrayList a = new ArrayList();
+	Resolver resolver = (Resolver) RT.READER_RESOLVER.deref();
 
 	for(; ;) {
 
-		Object form = read(r, false, READ_EOF, delim, READ_FINISHED, isRecursive, opts, pendingForms);
+		Object form = read(r, false, READ_EOF, delim, READ_FINISHED, isRecursive, opts, pendingForms,
+                           resolver);
 
 		if (form == READ_EOF) {
 			if (firstline < 0)
@@ -1343,7 +1528,7 @@ public static class ConditionalReader extends AFn {
 		for(; ;) {
 			if(result == READ_STARTED) {
 				// Read the next feature
-				form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms);
+				form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms, null);
 
 				if (form == READ_EOF) {
 					if (firstline < 0)
@@ -1361,7 +1546,7 @@ public static class ConditionalReader extends AFn {
 
 					//Read the form corresponding to the feature, and assign it to result if everything is kosher
 
-					form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms);
+					form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms, (Resolver) RT.READER_RESOLVER.deref());
 
 					if (form == READ_EOF) {
 						if (firstline < 0)
@@ -1382,7 +1567,7 @@ public static class ConditionalReader extends AFn {
 			// When we already have a result, or when the feature didn't match, discard the next form in the reader
 			try {
 				Var.pushThreadBindings(RT.map(RT.SUPPRESS_READ, RT.T));
-				form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms);
+				form = read(r, false, READ_EOF, ')', READ_FINISHED, true, opts, pendingForms, (Resolver) RT.READER_RESOLVER.deref());
 
 				if (form == READ_EOF) {
 					if (firstline < 0)
