@@ -354,6 +354,33 @@
   ;; throws an exception on failure
   (is (eval `(fn [] ~(CLJ1399. 1)))))
 
+(deftest CLJ-1250-this-clearing
+  (testing "clearing during try/catch/finally"
+    (let [closed-over-in-catch (let [x :foo]
+                                 (fn []
+                                   (try
+                                     (throw (Exception. "boom"))
+                                     (catch Exception e
+                                       x)))) ;; x should remain accessible to the fn
+
+          a (atom nil)
+          closed-over-in-finally (fn []
+                                   (try
+                                     :ret
+                                     (finally
+                                       (reset! a :run))))]
+      (is (= :foo (closed-over-in-catch)))
+      (is (= :ret (closed-over-in-finally)))
+      (is (= :run @a))))
+  (testing "no clearing when loop not in return context"
+    (let [x (atom 5)
+          bad (fn []
+                (loop [] (System/getProperties))
+                (swap! x dec)
+                (when (pos? @x)
+                  (recur)))]
+      (is (nil? (bad))))))
+
 (deftest CLJ-1586-lazyseq-literals-preserve-metadata
   (should-not-reflect (eval (list '.substring (with-meta (concat '(identity) '("foo")) {:tag 'String}) 0))))
 
@@ -395,3 +422,9 @@
       ;; eventually call `load` and reset called?.
       (require 'clojure.repl :reload))
     (is @called?)))
+
+(deftest clj-1714
+  (testing "CLJ-1714 Classes shouldn't have their static initialisers called simply by type hinting or importing"
+    ;; ClassWithFailingStaticInitialiser will throw if its static initialiser is called
+    (is (eval '(fn [^compilation.ClassWithFailingStaticInitialiser c])))
+    (is (eval '(import (compilation ClassWithFailingStaticInitialiser))))))
